local srcNames = {};
local srcClasses = {};
local healingDone = {};
local overhealingDone = {};
local absorbedDone = {};
local criticalDone = {};
local lastShotTime = {};
local combatStartTime = {};

local events = {
    RANGE_HEAL = true,
    SPELL_HEAL = true,
    SPELL_PERIODIC_HEAL = true,
    SPELL_BUILDING_HEAL = true,
}


function MistWeaver_InitStatistics()

    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("PLAYER_REGEN_ENABLED"); 
    frame:RegisterEvent("PLAYER_REGEN_DISABLED"); 
    frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED"); 
    
    frame:HookScript("OnEvent", MistWeaver_StatisticsOnEvent); 
end

function MistWeaver_StatisticsOnEvent(self, event, ...)
    if (event == "PLAYER_REGEN_ENABLED") then
        MistWeaver_Statistics_StopCombat();
    end
    if (event == "PLAYER_REGEN_DISABLED") then
        MistWeaver_Statistics_StartCombat();
    end
    if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
        local timestamp, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId, spellName, spellSchool, amount, overhealing, absorbed, critical = ...;
        MistWeaver_Statistics_Calculate(timestamp, eventType, hideCaster, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, overhealing, absorbed, critical);
    end
end

function MistWeaver_Statistics_StartCombat()
    srcNames = {};
    srcClasses = {};
    healingDone = {};
    overhealingDone = {};
    absorbedDone = {};
    criticalDone = {};
    lastShotTime = {};
    combatStartTime = {};
end

function MistWeaver_Statistics_StopCombat()
end

function MistWeaver_Statistics_Show()
    local ordered = {};
    
    for index, srcName in pairs(srcNames) do          
        local healingDoneValue = healingDone[srcName] or 0;
        local overhealingDoneValue = overhealingDone[srcName] or 0;
        
        local netto = healingDoneValue - overhealingDoneValue;
        
        ordered[netto] = {name = srcName, healing = healingDoneValue, overhealing = overhealingDoneValue};
    end 
    
    mw_info("MistWeaver "..STATISTICS);
    
    local color;
    
    for netto, data in mw_pairsByKeysDesc(ordered) do
        color = "909090";
        
        if (srcClasses[data.name]) then
            classColor = RAID_CLASS_COLORS[srcClasses[data.name]];
            
            if (classColor) then
                color = mw_rgb2hex(classColor.r, classColor.g, classColor.b);
            end
        end
               
        mw_info("  - |cff"..color..data.name..FONT_COLOR_CODE_CLOSE.." - healing: "..BreakUpLargeNumbers(netto).."    (amount: "..BreakUpLargeNumbers(data.healing)..", overhealing: "..BreakUpLargeNumbers(data.overhealing)..")");
    end
end

function MistWeaver_Statistics_Calculate(timestamp, eventType, hideCaster, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, overhealing, absorbed, critical)
    if (InCombatLockdown() == nil) then
        return;
    end
    
    if (not srcName) then
        return;
    end
    
    if (not events[eventType]) then
        return;
    end
    
    if (not srcNames[srcName]) then
        srcNames[srcName] = srcName;
    end
    
    if (not srcClasses[srcName]) then
        local locClass, engClass, locRace, engRace, gender = GetPlayerInfoByGUID(srcGUID);
        srcClasses[srcName] = engClass;
    end
    
    lastShotTime[srcName] = GetTime();
    
    -- collect values   
    if (amount and amount > 0) then
        local dd = (healingDone[srcName] or 0) + amount;
        healingDone[srcName] = dd;
        
        if (not combatStartTime[srcName]) then
            combatStartTime[srcName] = GetTime();
        end
    end
    
    if (overhealing and overhealing > 0) then
        local dd = (overhealingDone[srcName] or 0) + overhealing;
        overhealingDone[srcName] = dd;
    end
    
    if (absorbed and absorbed > 0) then
        local dd = (absorbedDone[srcName] or 0) + absorbed;
        absorbedDone[srcName] = dd;
    end
    
    if (critical and critical > 0) then
        local dd = (criticalDone[srcName] or 0) + critical;
        criticalDone[srcName] = dd;
    end
end